@@ -1,7 +1,10 @@
 # frozen_string_literal: true
+
 require 'timeout'
 require 'nokogiri'
 require 'xpath'
+require 'forwardable'
+require 'capybara/config'
 
 module Capybara
   class CapybaraError < StandardError; end
@@ -20,15 +23,37 @@ module Capybara
   class ReadOnlyElementError < CapybaraError; end
 
   class << self
-    attr_reader :app_host, :default_host
-    attr_accessor :asset_host, :run_server, :always_include_port
-    attr_accessor :server_port, :exact, :match, :exact_options, :visible_text_only, :enable_aria_label
-    attr_accessor :default_selector, :default_max_wait_time, :ignore_hidden_elements
-    attr_accessor :save_path, :wait_on_first_by_default, :automatic_label_click, :automatic_reload
-    attr_accessor :reuse_server, :raise_server_errors, :server_errors
-    attr_writer :default_driver, :current_driver, :javascript_driver, :session_name, :server_host
-    attr_reader :save_and_open_page_path
-    attr_accessor :app
+    extend Forwardable
+
+    # DelegateCapybara global configurations
+    # @!method app
+    #   See {Capybara.configure}
+    # @!method reuse_server
+    #   See {Capybara.configure}
+    # @!method threadsafe
+    #   See {Capybara.configure}
+    # @!method server
+    #   See {Capybara.configure}
+    # @!method default_driver
+    #   See {Capybara.configure}
+    # @!method javascript_driver
+    #   See {Capybara.configure}
+    Config::OPTIONS.each do |method|
+      def_delegators :config, method, "#{method}="
+    end
+
+    # Delegate Capybara global configurations
+    # @!method default_selector
+    #   See {Capybara.configure}
+    # @!method default_max_wait_time
+    #   See {Capybara.configure}
+    # @!method app_host
+    #   See {Capybara.configure}
+    # @!method always_include_port
+    #   See {Capybara.configure}
+    SessionConfig::OPTIONS.each do |method|
+      def_delegators :config, method, "#{method}="
+    end
 
     ##
     #
@@ -42,7 +67,7 @@ module Capybara
     # === Configurable options
     #
     # [app_host = String/nil]             The default host to use when giving a relative URL to visit, must be a valid URL e.g. http://www.example.com
-    # [always_include_port = Boolean]     Whether the Rack server's port should automatically be inserted into every visited URL (Default: false)
+    # [always_include_port = Boolean]     Whether the Rack server's port should automatically be inserted into every visited URL unless another port is explicitly specified (Default: false)
     # [asset_host = String]               Where dynamic assets are hosted - will be prepended to relative asset locations if present (Default: nil)
     # [run_server = Boolean]              Whether to start a Rack server for the given Rack app (Default: true)
     # [raise_server_errors = Boolean]     Should errors raised in the server be raised in the tests? (Default: true)
@@ -52,10 +77,16 @@ module Capybara
     # [ignore_hidden_elements = Boolean]  Whether to ignore hidden elements on the page (Default: true)
     # [automatic_reload = Boolean]        Whether to automatically reload elements as Capybara is waiting (Default: true)
     # [save_path = String]  Where to put pages saved through save_(page|screenshot), save_and_open_(page|screenshot) (Default: Dir.pwd)
-    # [wait_on_first_by_default = Boolean]   Whether Node#first defaults to Capybara waiting behavior for at least 1 element to match (Default: false)
     # [automatic_label_click = Boolean]   Whether Node#choose, Node#check, Node#uncheck will attempt to click the associated label element if the checkbox/radio button are non-visible (Default: false)
     # [enable_aria_label = Boolean]  Whether fields, links, and buttons will match against aria-label attribute (Default: false)
     # [reuse_server = Boolean]  Reuse the server thread between multiple sessions using the same app object (Default: true)
+    # [threadsafe = Boolean]  Whether sessions can be configured individually (Default: false)
+    # [server = Symbol]  The name of the registered server to use when running the app under test (Default: :webrick)
+    # [default_set_options = Hash]  The default options passed to Node::set (Default: {})
+    # [test_id = Symbol/String/nil] Optional attribute to match locator aginst with builtin selectors along with id (Default: nil)
+    # [predicates_wait = Boolean]  Whether Capybaras predicate matchers use waiting behavior by default (Default: true)
+    # [default_normalize_ws = Boolean] Whether text predicates and matchers use normalize whitespace behaviour (Default: false)
+    #
     # === DSL Options
     #
     # when using capybara/dsl, the following options are also available:
@@ -64,7 +95,7 @@ module Capybara
     # [javascript_driver = Symbol]        The name of a driver to use for JavaScript enabled tests. (Default: :selenium)
     #
     def configure
-      yield self
+      yield config
     end
 
     ##
@@ -98,7 +129,6 @@ module Capybara
     # @yieldparam [<Rack>] app                The rack application that this server will contain.
     # @yieldparam port                        The port number the server should listen on
     # @yieldparam host                        The host/ip to bind to
-    # @yieldreturn [Capybara::Driver::Base]   A Capybara driver instance
     #
     def register_server(name, &block)
       servers[name.to_sym] = block
@@ -139,13 +169,14 @@ module Capybara
     ##
     #
     # Modify a selector previously created by {Capybara.add_selector}.
-    # For example modifying the :button selector to also find divs styled
-    # to look like buttons might look like this
+    # For example, adding a new filter to the :button selector to filter based on
+    # button style (a class) might look like this
     #
     #     Capybara.modify_selector(:button) do
-    #       xpath { |locator| XPath::HTML.button(locator).or(XPath::css('div.btn')[XPath::string.n.is(locator)]) }
+    #       filter (:style, valid_values: [:primary, :secondary]) { |node, style| node[:class].split.include? "btn-#{style}" }
     #     end
     #
+    #
     # @param [Symbol] name    The name of the selector to modify
     # @yield                  A block executed in the context of the existing {Capybara::Selector}
     #
@@ -161,47 +192,6 @@ module Capybara
       @servers ||= {}
     end
 
-    ##
-    #
-    # Register a proc that Capybara will call to run the Rack application.
-    #
-    #     Capybara.server do |app, port, host|
-    #       require 'rack/handler/mongrel'
-    #       Rack::Handler::Mongrel.run(app, :Port => port)
-    #     end
-    #
-    # By default, Capybara will try to run webrick.
-    #
-    # @yield [app, port, host]      This block receives a rack app, port, and host/ip and should run a Rack handler
-    #
-    def server(&block)
-      if block_given?
-        warn "DEPRECATED: Passing a block to Capybara::server is deprecated, please use Capybara::register_server instead"
-        @server = block
-      else
-        @server
-      end
-    end
-
-    ##
-    #
-    # Set the server to use.
-    #
-    #     Capybara.server = :webrick
-    #
-    # @param [Symbol] name     Name of the server type to use
-    # @see register_server
-    #
-    def server=(name)
-      @server = if name.respond_to? :call
-        name
-      else
-        servers[name.to_sym]
-      end
-    end
-
-    ##
-    #
     # Wraps the given string, which should contain an HTML document or fragment
     # in a {Capybara::Node::Simple} which exposes all {Capybara::Node::Matchers},
     # {Capybara::Node::Finders} and {Capybara::Node::DocumentMatchers}. This allows you to query
@@ -243,18 +233,10 @@ module Capybara
     # manually.
     #
     # @param [Rack Application] app    The rack application to run
-    # @param [Fixnum] port              The port to run the application on
+    # @param [Integer] port              The port to run the application on
     #
     def run_default_server(app, port)
-      servers[:webrick].call(app, port, server_host)
-    end
-
-    ##
-    #
-    # @return [Symbol]    The name of the driver to use by default
-    #
-    def default_driver
-      @default_driver || :rack_test
+      servers[:puma].call(app, port, server_host)
     end
 
     ##
@@ -262,16 +244,20 @@ module Capybara
     # @return [Symbol]    The name of the driver currently in use
     #
     def current_driver
-      @current_driver || default_driver
+      if threadsafe
+        Thread.current['capybara_current_driver']
+      else
+        @current_driver
+      end || default_driver
     end
     alias_method :mode, :current_driver
 
-    ##
-    #
-    # @return [Symbol]    The name of the driver used when JavaScript is needed
-    #
-    def javascript_driver
-      @javascript_driver || :selenium
+    def current_driver=(name)
+      if threadsafe
+        Thread.current['capybara_current_driver'] = name
+      else
+        @current_driver = name
+      end
     end
 
     ##
@@ -279,7 +265,7 @@ module Capybara
     # Use the default driver as the current driver
     #
     def use_default_driver
-      @current_driver = nil
+      self.current_driver = nil
     end
 
     ##
@@ -291,15 +277,7 @@ module Capybara
       Capybara.current_driver = driver
       yield
     ensure
-      @current_driver = previous_driver
-    end
-
-    ##
-    #
-    # @return [String]    The IP address bound by default server
-    #
-    def server_host
-      @server_host || '127.0.0.1'
+      self.current_driver = previous_driver
     end
 
     ##
@@ -321,7 +299,7 @@ module Capybara
     # @return [Capybara::Session]     The currently used session
     #
     def current_session
-      session_pool["#{current_driver}:#{session_name}:#{app.object_id}"] ||= Capybara::Session.new(current_driver, app)
+      specified_session || session_pool["#{current_driver}:#{session_name}:#{app.object_id}"]
     end
 
     ##
@@ -330,8 +308,8 @@ module Capybara
     # as cookies.
     #
     def reset_sessions!
-      #reset in reverse so sessions that started servers are reset last
-      session_pool.reverse_each { |mode, session| session.reset! }
+      # reset in reverse so sessions that started servers are reset last
+      session_pool.reverse_each { |_mode, session| session.reset! }
     end
     alias_method :reset!, :reset_sessions!
 
@@ -342,19 +320,42 @@ module Capybara
     # @return [Symbol]    The name of the currently used session.
     #
     def session_name
-      @session_name ||= :default
+      if threadsafe
+        Thread.current['capybara_session_name'] ||= :default
+      else
+        @session_name ||= :default
+      end
+    end
+
+    def session_name=(name)
+      if threadsafe
+        Thread.current['capybara_session_name'] = name
+      else
+        @session_name = name
+      end
     end
 
     ##
     #
-    # Yield a block using a specific session name.
+    # Yield a block using a specific session name or Capybara::Session instance.
     #
-    def using_session(name)
-      previous_session_name = self.session_name
-      self.session_name = name
+    def using_session(name_or_session)
+      previous_session_info = {
+        specified_session: specified_session,
+        session_name: session_name,
+        current_driver: current_driver,
+        app: app
+      }
+      self.specified_session = self.session_name = nil
+      if name_or_session.is_a? Capybara::Session
+        self.specified_session = name_or_session
+      else
+        self.session_name = name_or_session
+      end
       yield
     ensure
-      self.session_name = previous_session_name
+      self.session_name, self.specified_session = previous_session_info.values_at(:session_name, :specified_session)
+      self.current_driver, self.app = previous_session_info.values_at(:current_driver, :app) if threadsafe
     end
 
     ##
@@ -364,62 +365,53 @@ module Capybara
     # @param [String] html              The raw html
     # @return [Nokogiri::HTML::Document]      HTML document
     #
-    def HTML(html)
-      Nokogiri::HTML(html).tap do |document|
-        document.xpath('//textarea').each do |textarea|
-          textarea.content=textarea.content.sub(/\A\n/,'')
+    def HTML(html) # rubocop:disable Naming/MethodName
+      if Nokogiri.respond_to?(:HTML5) # Nokogumbo installed
+        Nokogiri::HTML5(html).tap do |document|
+          document.xpath('//textarea').each do |textarea|
+            # The Nokogumbo HTML5 parser already returns spec compliant contents
+            textarea['_capybara_raw_value'] = textarea.content
+          end
+        end
+      else
+        Nokogiri::HTML(html).tap do |document|
+          document.xpath('//textarea').each do |textarea|
+            textarea['_capybara_raw_value'] = textarea.content.sub(/\A\n/, '')
+          end
         end
       end
     end
 
-    # @deprecated Use default_max_wait_time instead
-    def default_wait_time
-      deprecate('default_wait_time', 'default_max_wait_time', true)
-      default_max_wait_time
-    end
-
-    # @deprecated Use default_max_wait_time= instead
-    def default_wait_time=(t)
-      deprecate('default_wait_time=', 'default_max_wait_time=')
-      self.default_max_wait_time = t
-    end
-
-    def save_and_open_page_path=(path)
-      warn "DEPRECATED: #save_and_open_page_path is deprecated, please use #save_path instead. \n"\
-           "Note: Behavior is slightly different with relative paths - see documentation" unless path.nil?
-      @save_and_open_page_path = path
-    end
-
-    def app_host=(url)
-      raise ArgumentError.new("Capybara.app_host should be set to a url (http://www.example.com)") unless url.nil? || (url =~ URI::regexp)
-      @app_host = url
-    end
-
-    def default_host=(url)
-      raise ArgumentError.new("Capybara.default_host should be set to a url (http://www.example.com)") unless url.nil? || (url =~ URI::regexp)
-      @default_host = url
-    end
-
-    def included(base)
-      base.send(:include, Capybara::DSL)
-      warn "`include Capybara` is deprecated. Please use `include Capybara::DSL` instead."
-    end
-
-    def reuse_server=(bool)
-      warn "Capybara.reuse_server == false is a BETA feature and may change in a future version" unless bool
-      @reuse_server = bool
-    end
-
-    def deprecate(method, alternate_method, once=false)
-      @deprecation_notified ||= {}
-      warn "DEPRECATED: ##{method} is deprecated, please use ##{alternate_method} instead" unless once and @deprecation_notified[method]
-      @deprecation_notified[method]=true
+    def session_options
+      config.session_options
     end
 
   private
 
+    def config
+      @config ||= Capybara::Config.new
+    end
+
     def session_pool
-      @session_pool ||= {}
+      @session_pool ||= Hash.new do |hash, name|
+        hash[name] = Capybara::Session.new(current_driver, app)
+      end
+    end
+
+    def specified_session
+      if threadsafe
+        Thread.current['capybara_specified_session']
+      else
+        @specified_session ||= nil
+      end
+    end
+
+    def specified_session=(session)
+      if threadsafe
+        Thread.current['capybara_specified_session'] = session
+      else
+        @specified_session = session
+      end
     end
   end
 
@@ -445,7 +437,9 @@ module Capybara
   require 'capybara/queries/title_query'
   require 'capybara/queries/current_path_query'
   require 'capybara/queries/match_query'
-  require 'capybara/query'
+  require 'capybara/queries/ancestor_query'
+  require 'capybara/queries/sibling_query'
+  require 'capybara/queries/style_query'
 
   require 'capybara/node/finders'
   require 'capybara/node/matchers'
@@ -469,19 +463,40 @@ module Capybara
   require 'capybara/selenium/driver'
 end
 
-Capybara.register_server :default do |app, port, host|
+Capybara.register_server :default do |app, port, _host|
   Capybara.run_default_server(app, port)
 end
 
-Capybara.register_server :webrick do |app, port, host|
+Capybara.register_server :webrick do |app, port, host, **options|
   require 'rack/handler/webrick'
-  Rack::Handler::WEBrick.run(app, Host: host, Port: port, AccessLog: [], Logger: WEBrick::Log::new(nil, 0))
+  options = { Host: host, Port: port, AccessLog: [], Logger: WEBrick::Log.new(nil, 0) }.merge(options)
+  Rack::Handler::WEBrick.run(app, options)
 end
 
-Capybara.register_server :puma do |app, port, host|
-  require 'puma'
-  Puma::Server.new(app).tap do |s|
-    s.add_tcp_listener host, port
+Capybara.register_server :puma do |app, port, host, **options|
+  begin
+    require 'rack/handler/puma'
+  rescue LoadError
+    raise LoadError, 'Capybara is unable to load `puma` for its server, please add `puma` to your project or specify a different server via something like `Capybara.server = :webrick`.'
+  else
+    unless Rack::Handler::Puma.respond_to?(:config)
+      raise LoadError, 'Capybara requires `puma` version 3.8.0 or higher, please upgrade `puma` or register and specify your own server block'
+    end
+  end
+  # If we just run the Puma Rack handler it installs signal handlers which prevent us from being able to interrupt tests.
+  # Therefore construct and run the Server instance ourselves.
+  # Rack::Handler::Puma.run(app, { Host: host, Port: port, Threads: "0:4", workers: 0, daemon: false }.merge(options))
+  options = { Host: host, Port: port, Threads: '0:4', workers: 0, daemon: false }.merge(options)
+  conf = Rack::Handler::Puma.config(app, options)
+  events = conf.options[:Silent] ? ::Puma::Events.strings : ::Puma::Events.stdio
+
+  events.log 'Capybara starting Puma...'
+  events.log "* Version #{Puma::Const::PUMA_VERSION} , codename: #{Puma::Const::CODE_NAME}"
+  events.log "* Min threads: #{conf.options[:min_threads]}, max threads: #{conf.options[:max_threads]}"
+
+  Puma::Server.new(conf.app, events, conf.options).tap do |s|
+    s.binder.parse conf.options[:binds], s.events
+    s.min_threads, s.max_threads = conf.options[:min_threads], conf.options[:max_threads]
   end.run.join
 end
 
@@ -492,17 +507,21 @@ Capybara.configure do |config|
   config.default_selector = :css
   config.default_max_wait_time = 2
   config.ignore_hidden_elements = true
-  config.default_host = "http://www.example.com"
+  config.default_host = 'http://www.example.com'
   config.automatic_reload = true
   config.match = :smart
   config.exact = false
+  config.exact_text = false
   config.raise_server_errors = true
   config.server_errors = [StandardError]
   config.visible_text_only = false
-  config.wait_on_first_by_default = false
   config.automatic_label_click = false
   config.enable_aria_label = false
   config.reuse_server = true
+  config.default_set_options = {}
+  config.test_id = nil
+  config.predicates_wait = true
+  config.default_normalize_ws = false
 end
 
 Capybara.register_driver :rack_test do |app|
@@ -513,3 +532,21 @@ Capybara.register_driver :selenium do |app|
   Capybara::Selenium::Driver.new(app)
 end
 
+Capybara.register_driver :selenium_headless do |app|
+  Capybara::Selenium::Driver.load_selenium
+  browser_options = ::Selenium::WebDriver::Firefox::Options.new
+  browser_options.args << '-headless'
+  Capybara::Selenium::Driver.new(app, browser: :firefox, options: browser_options)
+end
+
+Capybara.register_driver :selenium_chrome do |app|
+  Capybara::Selenium::Driver.new(app, browser: :chrome)
+end
+
+Capybara.register_driver :selenium_chrome_headless do |app|
+  Capybara::Selenium::Driver.load_selenium
+  browser_options = ::Selenium::WebDriver::Chrome::Options.new
+  browser_options.args << '--headless'
+  browser_options.args << '--disable-gpu' if Gem.win_platform?
+  Capybara::Selenium::Driver.new(app, browser: :chrome, options: browser_options)
+end
