@@ -1,4 +1,8 @@
+# frozen_string_literal: true
+
 class Capybara::Driver::Base
+  attr_writer :session
+
   def current_url
     raise NotImplementedError
   end
@@ -7,6 +11,10 @@ class Capybara::Driver::Base
     raise NotImplementedError
   end
 
+  def refresh
+    raise NotImplementedError
+  end
+
   def find_xpath(query)
     raise NotImplementedError
   end
@@ -27,15 +35,19 @@ class Capybara::Driver::Base
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#go_forward'
   end
 
-  def execute_script(script)
+  def execute_script(script, *args)
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#execute_script'
   end
 
-  def evaluate_script(script)
+  def evaluate_script(script, *args)
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#evaluate_script'
   end
 
-  def save_screenshot(path, options={})
+  def evaluate_async_script(script, *args)
+    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#evaluate_script_asnyc'
+  end
+
+  def save_screenshot(path, **options)
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#save_screenshot'
   end
 
@@ -47,8 +59,22 @@ class Capybara::Driver::Base
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#status_code'
   end
 
-  def within_frame(frame_handle)
-    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#within_frame'
+  ##
+  #
+  # @param frame [Capybara::Node::Element, :parent, :top]  The iframe element to switch to
+  #
+  def switch_to_frame(frame)
+    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#switch_to_frame'
+  end
+
+  def frame_title
+    find_xpath('/html/head/title').map(&:all_text).first.to_s
+  end
+
+  def frame_url
+    evaluate_script('document.location.href')
+  rescue Capybara::NotSupportedByDriverError
+    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#frame_title'
   end
 
   def current_window_handle
@@ -64,7 +90,11 @@ class Capybara::Driver::Base
   end
 
   def maximize_window(handle)
-    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#maximize_current_window'
+    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#maximize_window'
+  end
+
+  def fullscreen_window(handle)
+    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#fullscreen_window'
   end
 
   def close_window(handle)
@@ -83,15 +113,10 @@ class Capybara::Driver::Base
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#switch_to_window'
   end
 
-  def within_window(locator)
-    raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#within_window'
-  end
-
   def no_such_window_error
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#no_such_window_error'
   end
-  
-  
+
   ##
   #
   # Execute the block, and then accept the modal opened.
@@ -102,7 +127,7 @@ class Capybara::Driver::Base
   # @return [String]  the message shown in the modal
   # @raise [Capybara::ModalNotFound]  if modal dialog hasn't been found
   #
-  def accept_modal(type, options={}, &blk)
+  def accept_modal(type, **options, &blk)
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#accept_modal'
   end
 
@@ -115,7 +140,7 @@ class Capybara::Driver::Base
   # @return [String]  the message shown in the modal
   # @raise [Capybara::ModalNotFound]  if modal dialog hasn't been found
   #
-  def dismiss_modal(type, options={}, &blk)
+  def dismiss_modal(type, **options, &blk)
     raise Capybara::NotSupportedByDriverError, 'Capybara::Driver::Base#dismiss_modal'
   end
 
@@ -127,14 +152,19 @@ class Capybara::Driver::Base
     false
   end
 
-  def reset!
-  end
+  def reset!; end
 
   def needs_server?
     false
   end
 
-  def browser_initialized?
-    true
+  def session_options
+    session&.config || Capybara.session_options
+  end
+
+private
+
+  def session
+    @session ||= nil
   end
 end
