@@ -1,7 +1,7 @@
 # frozen_string_literal: true
+
 module Capybara
   module Node
-
     ##
     #
     # A {Capybara::Document} represents an HTML document. Any operation
@@ -20,10 +20,14 @@ module Capybara
       #
       # @return [String]    The text of the document
       #
-      def text(type=nil)
-        find(:xpath, '/html').text(type)
+      def text(type = nil, normalize_ws: false)
+        find(:xpath, '/html').text(type, normalize_ws: normalize_ws)
       end
 
+      ##
+      #
+      # @return [String]    The title of the document
+      #
       def title
         session.driver.title
       end
