@@ -1,9 +1,9 @@
 # frozen_string_literal: true
+
 require 'capybara/session/matchers'
 require 'addressable/uri'
 
 module Capybara
-
   ##
   #
   # The Session class represents a single user's interaction with the system. The Session can use
@@ -17,8 +17,16 @@ module Capybara
   #     session = Capybara::Session.new(:culerity)
   #     session.visit('http://www.google.com')
   #
+  # When Capybara.threadsafe == true the sessions options will be initially set to the
+  # current values of the global options and a configuration block can be passed to the session initializer.
+  # For available options see {Capybara::SessionConfig::OPTIONS}
+  #
+  #     session = Capybara::Session.new(:driver, MyRackApp) do |config|
+  #       config.app_host = "http://my_host.dev"
+  #     end
+  #
   # Session provides a number of methods for controlling the navigation of the page, such as +visit+,
-  # +current_path, and so on. It also delegate a number of methods to a Capybara::Document, representing
+  # +current_path, and so on. It also delegates a number of methods to a Capybara::Document, representing
   # the current HTML document. This allows interaction:
   #
   #     session.fill_in('q', with: 'Capybara')
@@ -30,60 +38,70 @@ module Capybara
   class Session
     include Capybara::SessionMatchers
 
-    NODE_METHODS = [
-      :all, :first, :attach_file, :text, :check, :choose,
-      :click_link_or_button, :click_button, :click_link, :field_labeled,
-      :fill_in, :find, :find_all, :find_button, :find_by_id, :find_field, :find_link,
-      :has_content?, :has_text?, :has_css?, :has_no_content?, :has_no_text?,
-      :has_no_css?, :has_no_xpath?, :resolve, :has_xpath?, :select, :uncheck,
-      :has_link?, :has_no_link?, :has_button?, :has_no_button?, :has_field?,
-      :has_no_field?, :has_checked_field?, :has_unchecked_field?,
-      :has_no_table?, :has_table?, :unselect, :has_select?, :has_no_select?,
-      :has_selector?, :has_no_selector?, :click_on, :has_no_checked_field?,
-      :has_no_unchecked_field?, :query, :assert_selector, :assert_no_selector,
-      :refute_selector, :assert_text, :assert_no_text
-    ]
+    NODE_METHODS = %i[
+      all first attach_file text check choose
+      click_link_or_button click_button click_link
+      fill_in find find_all find_button find_by_id find_field find_link
+      has_content? has_text? has_css? has_no_content? has_no_text?
+      has_no_css? has_no_xpath? has_xpath? select uncheck
+      has_link? has_no_link? has_button? has_no_button? has_field?
+      has_no_field? has_checked_field? has_unchecked_field?
+      has_no_table? has_table? unselect has_select? has_no_select?
+      has_selector? has_no_selector? click_on has_no_checked_field?
+      has_no_unchecked_field? query assert_selector assert_no_selector
+      assert_all_of_selectors assert_none_of_selectors assert_any_of_selectors
+      refute_selector assert_text assert_no_text
+    ].freeze
     # @api private
-    DOCUMENT_METHODS = [
-      :title, :assert_title, :assert_no_title, :has_title?, :has_no_title?
-    ]
-    SESSION_METHODS = [
-      :body, :html, :source, :current_url, :current_host, :current_path,
-      :execute_script, :evaluate_script, :visit, :go_back, :go_forward,
-      :within, :within_fieldset, :within_table, :within_frame, :current_window,
-      :windows, :open_new_window, :switch_to_window, :within_window, :window_opened_by,
-      :save_page, :save_and_open_page, :save_screenshot,
-      :save_and_open_screenshot, :reset_session!, :response_headers,
-      :status_code, :current_scope,
-      :assert_current_path, :assert_no_current_path, :has_current_path?, :has_no_current_path?
-    ] + DOCUMENT_METHODS
-    MODAL_METHODS = [
-      :accept_alert, :accept_confirm, :dismiss_confirm, :accept_prompt,
-      :dismiss_prompt
-    ]
+    DOCUMENT_METHODS = %i[
+      title assert_title assert_no_title has_title? has_no_title?
+    ].freeze
+    SESSION_METHODS = %i[
+      body html source current_url current_host current_path
+      execute_script evaluate_script visit refresh go_back go_forward
+      within within_element within_fieldset within_table within_frame switch_to_frame
+      current_window windows open_new_window switch_to_window within_window window_opened_by
+      save_page save_and_open_page save_screenshot
+      save_and_open_screenshot reset_session! response_headers
+      status_code current_scope
+      assert_current_path assert_no_current_path has_current_path? has_no_current_path?
+    ].freeze + DOCUMENT_METHODS
+    MODAL_METHODS = %i[
+      accept_alert accept_confirm dismiss_confirm accept_prompt dismiss_prompt
+    ].freeze
     DSL_METHODS = NODE_METHODS + SESSION_METHODS + MODAL_METHODS
 
     attr_reader :mode, :app, :server
     attr_accessor :synchronized
 
-    def initialize(mode, app=nil)
+    def initialize(mode, app = nil)
+      raise TypeError, 'The second parameter to Session::new should be a rack app if passed.' if app && !app.respond_to?(:call)
+
+      @@instance_created = true
       @mode = mode
       @app = app
-      if Capybara.run_server and @app and driver.needs_server?
-        @server = Capybara::Server.new(@app).boot
-      else
-        @server = nil
+      if block_given?
+        raise 'A configuration block is only accepted when Capybara.threadsafe == true' unless Capybara.threadsafe
+
+        yield config
+      end
+      @server = if config.run_server && @app && driver.needs_server?
+        server_options = { port: config.server_port, host: config.server_host, reportable_errors: config.server_errors }
+        server_options[:extra_middleware] = [Capybara::Server::AnimationDisabler] if config.disable_animation
+        Capybara::Server.new(@app, server_options).boot
       end
       @touched = false
     end
 
     def driver
       @driver ||= begin
-        unless Capybara.drivers.has_key?(mode)
-          other_drivers = Capybara.drivers.keys.map { |key| key.inspect }
+        unless Capybara.drivers.key?(mode)
+          other_drivers = Capybara.drivers.keys.map(&:inspect)
           raise Capybara::DriverNotFoundError, "no driver called #{mode.inspect} was found, available drivers: #{other_drivers.join(', ')}"
         end
-        Capybara.drivers[mode].call(app)
+        driver = Capybara.drivers[mode].call(app)
+        driver.session = self if driver.respond_to?(:session=)
+        driver
       end
     end
 
@@ -110,7 +128,7 @@ module Capybara
         driver.reset!
         @touched = false
       end
-      @server.wait_for_pending_requests if @server
+      @server&.wait_for_pending_requests
       raise_server_error!
     end
     alias_method :cleanup!, :reset!
@@ -121,17 +139,19 @@ module Capybara
     # Raise errors encountered in the server
     #
     def raise_server_error!
-      if Capybara.raise_server_errors and @server and @server.error
-        # Force an explanation for the error being raised as the exception cause
-        begin
-          raise CapybaraError, "Your application server raised an error - It has been raised in your test code because Capybara.raise_server_errors == true"
-        rescue CapybaraError
-          #needed to get the cause set correctly in JRuby -- otherwise we could just do raise @server.error
-          raise @server.error, @server.error.message, @server.error.backtrace
+      return unless @server&.error
+
+      # Force an explanation for the error being raised as the exception cause
+      begin
+        if config.raise_server_errors
+          raise CapybaraError, 'Your application server raised an error - It has been raised in your test code because Capybara.raise_server_errors == true'
         end
+      rescue CapybaraError
+        # needed to get the cause set correctly in JRuby -- otherwise we could just do raise @server.error
+        raise @server.error, @server.error.message, @server.error.backtrace
+      ensure
+        @server.reset_error!
       end
-    ensure
-      @server.reset_error! if @server
     end
 
     ##
@@ -170,11 +190,13 @@ module Capybara
     #
     def current_path
       # Addressable parsing is more lenient than URI
-      uri = Addressable::URI.parse(current_url)
+      uri = ::Addressable::URI.parse(current_url)
+
       # Addressable doesn't support opaque URIs - we want nil here
-      return nil if uri.scheme == "about"
-      path = uri.path
-      path if path and not path.empty?
+      return nil if uri&.scheme == 'about'
+
+      path = uri&.path
+      path unless path&.empty?
     end
 
     ##
@@ -224,28 +246,34 @@ module Capybara
       raise_server_error!
       @touched = true
 
-      visit_uri = URI.parse(visit_uri.to_s)
+      visit_uri = ::Addressable::URI.parse(visit_uri.to_s)
+      base_uri = ::Addressable::URI.parse(config.app_host || server_url)
 
-      uri_base = if @server
-        visit_uri.port = @server.port if Capybara.always_include_port && (visit_uri.port == visit_uri.default_port)
-        URI.parse(Capybara.app_host || "http://#{@server.host}:#{@server.port}")
-      else
-        Capybara.app_host && URI.parse(Capybara.app_host)
+      if base_uri && [nil, 'http', 'https'].include?(visit_uri.scheme)
+        if visit_uri.relative?
+          visit_uri_parts = visit_uri.to_hash.delete_if { |_k, value| value.nil? }
+
+          # Useful to people deploying to a subdirectory
+          # and/or single page apps where only the url fragment changes
+          visit_uri_parts[:path] = base_uri.path + visit_uri.path
+
+          visit_uri = base_uri.merge(visit_uri_parts)
+        end
+        adjust_server_port(visit_uri)
       end
 
-      # TODO - this is only for compatability with previous 2.x behavior that concatenated
-      # Capybara.app_host and a "relative" path - Consider removing in 3.0
-      # @abotalov brought up a good point about this behavior potentially being useful to people
-      # deploying to a subdirectory and/or single page apps where only the url fragment changes
-      if visit_uri.scheme.nil? && uri_base
-        visit_uri.path = uri_base.path + visit_uri.path
-      end
-
-      visit_uri = uri_base.merge(visit_uri) unless uri_base.nil?
-
       driver.visit(visit_uri.to_s)
     end
 
+    ##
+    #
+    # Refresh the page
+    #
+    def refresh
+      raise_server_error!
+      driver.refresh
+    end
+
     ##
     #
     # Move back a single entry in the browser's history.
@@ -269,7 +297,7 @@ module Capybara
     # block, any command to Capybara will be handled as though it were scoped
     # to the given element.
     #
-    #     within(:xpath, '//div[@id="delivery-address"]') do
+    #     within(:xpath, './/div[@id="delivery-address"]') do
     #       fill_in('Street', with: '12 Main Street')
     #     end
     #
@@ -298,14 +326,15 @@ module Capybara
     # @raise  [Capybara::ElementNotFound]      If the scope can't be found before time expires
     #
     def within(*args)
-      new_scope = if args.first.is_a?(Capybara::Node::Base) then args.first else find(*args) end
+      new_scope = args.first.respond_to?(:to_capybara_node) ? args.first.to_capybara_node : find(*args)
       begin
         scopes.push(new_scope)
-        yield
+        yield if block_given?
       ensure
         scopes.pop
       end
     end
+    alias_method :within_element, :within
 
     ##
     #
@@ -314,9 +343,7 @@ module Capybara
     # @param [String] locator    Id or legend of the fieldset
     #
     def within_fieldset(locator)
-      within :fieldset, locator do
-        yield
-      end
+      within(:fieldset, locator) { yield }
     end
 
     ##
@@ -326,8 +353,48 @@ module Capybara
     # @param [String] locator    Id or caption of the table
     #
     def within_table(locator)
-      within :table, locator do
-        yield
+      within(:table, locator) { yield }
+    end
+
+    ##
+    #
+    # Switch to the given frame
+    #
+    # If you use this method you are responsible for making sure you switch back to the parent frame when done in the frame changed to.
+    # Capybara::Session#within_frame is preferred over this method and should be used when possible.
+    # May not be supported by all drivers.
+    #
+    # @overload switch_to_frame(element)
+    #   @param [Capybara::Node::Element]  iframe/frame element to switch to
+    # @overload switch_to_frame(:parent)
+    #   Switch to the parent frame
+    # @overload switch_to_frame(:top)
+    #   Switch to the top level document
+    #
+    def switch_to_frame(frame)
+      case frame
+      when Capybara::Node::Element
+        driver.switch_to_frame(frame)
+        scopes.push(:frame)
+      when :parent
+        if scopes.last != :frame
+          raise Capybara::ScopeError, "`switch_to_frame(:parent)` cannot be called from inside a descendant frame's "\
+                                      '`within` block.'
+        end
+        scopes.pop
+        driver.switch_to_frame(:parent)
+      when :top
+        idx = scopes.index(:frame)
+        if idx
+          if scopes.slice(idx..-1).any? { |scope| ![:frame, nil].include?(scope) }
+            raise Capybara::ScopeError, "`switch_to_frame(:top)` cannot be called from inside a descendant frame's "\
+                                        '`within` block.'
+          end
+          scopes.slice!(idx..-1)
+          driver.switch_to_frame(:top)
+        end
+      else
+        raise ArgumentError, 'You must provide a frame element, :parent, or :top when calling switch_to_frame'
       end
     end
 
@@ -338,45 +405,18 @@ module Capybara
     #
     # @overload within_frame(element)
     #   @param [Capybara::Node::Element]  frame element
-    # @overload within_frame(name)
-    #   @param [String] name           name/id of a frame
+    # @overload within_frame([kind = :frame], locator, **options)
+    #   @param [Symbol] kind      Optional selector type (:frame, :css, :xpath, etc.) - Defaults to :frame
+    #   @param [String] locator   The locator for the given selector kind.  For :frame this is the name/id of a frame/iframe element
     # @overload within_frame(index)
-    #   @param [Integer] index         index of a frame
-    #
-    def within_frame(locator)
-      scopes.push(nil)
-
-      #support older driver frame api for now
-      frame = case locator
-      when Capybara::Node::Element
-        locator
-      when String
-        find(:frame, locator)
-      when Integer
-        all(:frame, minimum: locator+1)[locator]
-      else
-        raise ArgumentError
-      end
-
+    #   @param [Integer] index         index of a frame (0 based)
+    def within_frame(*args)
+      switch_to_frame(_find_frame(*args))
       begin
-        driver.switch_to_frame(frame)
-        begin
-          yield
-        ensure
-          driver.switch_to_frame(:parent)
-        end
-      rescue Capybara::NotSupportedByDriverError
-        # Support older driver frame API for now
-        if driver.respond_to?(:within_frame)
-          driver.within_frame(frame) do
-            yield
-          end
-        else
-          raise
-        end
+        yield if block_given?
+      ensure
+        switch_to_frame(:parent)
       end
-    ensure
-      scopes.pop
     end
 
     ##
@@ -421,49 +461,23 @@ module Capybara
     #   @raise [Capybara::WindowError]     if no window matches given block
     # @overload switch_to_window(window)
     #   @param window [Capybara::Window]   window that should be switched to
-    #   @raise [Capybara::Driver::Base#no_such_window_error] if unexistent (e.g. closed) window was passed
+    #   @raise [Capybara::Driver::Base#no_such_window_error] if nonexistent (e.g. closed) window was passed
     #
     # @return [Capybara::Window]         window that has been switched to
-    # @raise [Capybara::ScopeError]        if this method is invoked inside `within`,
-    #   `within_frame` or `within_window` methods
+    # @raise [Capybara::ScopeError]        if this method is invoked inside `within` or
+    #   `within_frame` methods
     # @raise [ArgumentError]               if both or neither arguments were provided
     #
-    def switch_to_window(window = nil, options= {})
-      options, window = window, nil if window.is_a? Hash
+    def switch_to_window(window = nil, **options, &window_locator)
+      raise ArgumentError, '`switch_to_window` can take either a block or a window, not both' if window && block_given?
+      raise ArgumentError, '`switch_to_window`: either window or block should be provided' if !window && !block_given?
 
-      block_given = block_given?
-      if window && block_given
-        raise ArgumentError, "`switch_to_window` can take either a block or a window, not both"
-      elsif !window && !block_given
-        raise ArgumentError, "`switch_to_window`: either window or block should be provided"
-      elsif scopes.size > 1
-        raise Capybara::ScopeError, "`switch_to_window` is not supposed to be invoked from "\
-                                    "`within`'s, `within_frame`'s' or `within_window`'s' block."
+      unless scopes.last.nil?
+        raise Capybara::ScopeError, '`switch_to_window` is not supposed to be invoked from '\
+                                    '`within` or `within_frame` blocks.'
       end
 
-      if window
-        driver.switch_to_window(window.handle)
-        window
-      else
-        wait_time = Capybara::Queries::BaseQuery.wait(options)
-        document.synchronize(wait_time, errors: [Capybara::WindowError]) do
-          original_window_handle = driver.current_window_handle
-          begin
-            driver.window_handles.each do |handle|
-              driver.switch_to_window handle
-              if yield
-                return Window.new(self, handle)
-              end
-            end
-          rescue => e
-            driver.switch_to_window(original_window_handle)
-            raise e
-          else
-            driver.switch_to_window(original_window_handle)
-            raise Capybara::WindowError, "Could not find a window matching block/lambda"
-          end
-        end
-      end
+      _switch_to_window(window, options, &window_locator)
     end
 
     ##
@@ -476,53 +490,37 @@ module Capybara
     # @overload within_window(window) { do_something }
     #   @param window [Capybara::Window]       instance of `Capybara::Window` class
     #     that will be switched to
-    #   @raise [driver#no_such_window_error] if unexistent (e.g. closed) window was passed
+    #   @raise [driver#no_such_window_error] if nonexistent (e.g. closed) window was passed
     # @overload within_window(proc_or_lambda) { do_something }
     #   @param lambda [Proc]                  lambda. First window for which lambda
     #     returns a value other than false or nil will be switched to.
     #   @example
     #     within_window(->{ page.title == 'Page title' }) { click_button 'Submit' }
     #   @raise [Capybara::WindowError]         if no window matching lambda was found
-    # @overload within_window(string) { do_something }
-    #   @deprecated                            Pass window or lambda instead
-    #   @param [String]                        handle, name, url or title of the window
     #
-    # @raise [Capybara::ScopeError]        if this method is invoked inside `within`,
-    #   `within_frame` or `within_window` methods
+    # @raise [Capybara::ScopeError]        if this method is invoked inside `within_frame` method
     # @return                              value returned by the block
     #
-    def within_window(window_or_handle)
-      if window_or_handle.instance_of?(Capybara::Window)
-        original = current_window
-        switch_to_window(window_or_handle) unless original == window_or_handle
-        scopes << nil
-        begin
-          yield
-        ensure
-          @scopes.pop
-          switch_to_window(original) unless original == window_or_handle
+    def within_window(window_or_proc)
+      original = current_window
+      scopes << nil
+      begin
+        case window_or_proc
+        when Capybara::Window
+          _switch_to_window(window_or_proc) unless original == window_or_proc
+        when Proc
+          _switch_to_window { window_or_proc.call }
+        else
+          raise ArgumentError('`#within_window` requires a `Capybara::Window` instance or a lambda')
         end
-      elsif window_or_handle.is_a?(Proc)
-        original = current_window
-        switch_to_window { window_or_handle.call }
-        scopes << nil
+
         begin
-          yield
+          yield if block_given?
         ensure
-          @scopes.pop
-          switch_to_window(original)
-        end
-      else
-        offending_line = caller.first
-        file_line = offending_line.match(/^(.+?):(\d+)/)[0]
-        warn "DEPRECATION WARNING: Passing string argument to #within_window is deprecated. "\
-             "Pass window object or lambda. (called from #{file_line})"
-        begin
-          scopes << nil
-          driver.within_window(window_or_handle) { yield }
-        ensure
-          @scopes.pop
+          _switch_to_window(original) unless original == window_or_proc
         end
+      ensure
+        scopes.pop
       end
     end
 
@@ -532,21 +530,21 @@ module Capybara
     # It's better to use this method than `windows.last`
     # {https://dvcs.w3.org/hg/webdriver/raw-file/default/webdriver-spec.html#h_note_10 as order of windows isn't defined in some drivers}
     #
-    # @param options [Hash]
-    # @option options [Numeric] :wait (Capybara.default_max_wait_time) maximum wait time
-    # @return [Capybara::Window]       the window that has been opened within a block
-    # @raise [Capybara::WindowError]   if block passed to window hasn't opened window
-    #   or opened more than one window
+    # @overload window_opened_by(**options, &block)
+    #   @param options [Hash]
+    #   @option options [Numeric] :wait (Capybara.default_max_wait_time) maximum wait time
+    #   @return [Capybara::Window]       the window that has been opened within a block
+    #   @raise [Capybara::WindowError]   if block passed to window hasn't opened window
+    #     or opened more than one window
     #
-    def window_opened_by(options = {}, &block)
+    def window_opened_by(**options)
       old_handles = driver.window_handles
-      block.call
+      yield
 
-      wait_time = Capybara::Queries::BaseQuery.wait(options)
-      document.synchronize(wait_time, errors: [Capybara::WindowError]) do
+      synchronize_windows(options) do
         opened_handles = (driver.window_handles - old_handles)
         if opened_handles.size != 1
-          raise Capybara::WindowError, "block passed to #window_opened_by "\
+          raise Capybara::WindowError, 'block passed to #window_opened_by '\
                                        "opened #{opened_handles.size} windows instead of 1"
         end
         Window.new(self, opened_handles.first)
@@ -560,10 +558,11 @@ module Capybara
     # +evaluate_script+ whenever possible.
     #
     # @param [String] script   A string of JavaScript to execute
+    # @param args  Optional arguments that will be passed to the script.  Driver support for this is optional and types of objects supported may differ between drivers
     #
-    def execute_script(script)
+    def execute_script(script, *args)
       @touched = true
-      driver.execute_script(script)
+      driver.execute_script(script, *driver_args(args))
     end
 
     ##
@@ -575,9 +574,23 @@ module Capybara
     # @param  [String] script   A string of JavaScript to evaluate
     # @return [Object]          The result of the evaluated JavaScript (may be driver specific)
     #
-    def evaluate_script(script)
+    def evaluate_script(script, *args)
       @touched = true
-      driver.evaluate_script(script)
+      result = driver.evaluate_script(script.strip, *driver_args(args))
+      element_script_result(result)
+    end
+
+    ##
+    #
+    # Evaluate the given JavaScript and obtain the result from a callback function which will be passed as the last argument to the script.
+    #
+    # @param  [String] script   A string of JavaScript to evaluate
+    # @return [Object]          The result of the evaluated JavaScript (may be driver specific)
+    #
+    def evaluate_async_script(script, *args)
+      @touched = true
+      result = driver.evaluate_async_script(script, *driver_args(args))
+      element_script_result(result)
     end
 
     ##
@@ -585,19 +598,23 @@ module Capybara
     # Execute the block, accepting a alert.
     #
     # @!macro modal_params
-    #   @overload $0(text, options = {}, &blk)
+    #   Expects a block whose actions will trigger the display modal to appear
+    #   @example
+    #     $0 do
+    #       click_link('link that triggers appearance of system modal')
+    #     end
+    #   @overload $0(text, **options, &blk)
     #     @param text [String, Regexp]  Text or regex to match against the text in the modal.  If not provided any modal is matched
-    #   @overload $0(options = {}, &blk)
-    #   @option options [Numeric] :wait (Capybara.default_max_wait_time) Maximum time to wait for the modal to appear after executing the block.
+    #     @option options [Numeric] :wait (Capybara.default_max_wait_time) Maximum time to wait for the modal to appear after executing the block.
+    #     @yield Block whose actions will trigger the system modal
+    #   @overload $0(**options, &blk)
+    #     @option options [Numeric] :wait (Capybara.default_max_wait_time) Maximum time to wait for the modal to appear after executing the block.
+    #     @yield Block whose actions will trigger the system modal
     #   @return [String]  the message shown in the modal
     #   @raise [Capybara::ModalNotFound]  if modal dialog hasn't been found
     #
-    def accept_alert(text_or_options=nil, options={}, &blk)
-      text_or_options, options = nil, text_or_options if text_or_options.is_a?(Hash)
-      options[:text] ||= text_or_options unless text_or_options.nil?
-      options[:wait] ||= Capybara.default_max_wait_time
-
-      driver.accept_modal(:alert, options, &blk)
+    def accept_alert(text = nil, **options, &blk)
+      accept_modal(:alert, text, options, &blk)
     end
 
     ##
@@ -606,12 +623,8 @@ module Capybara
     #
     # @macro modal_params
     #
-    def accept_confirm(text_or_options=nil, options={}, &blk)
-      text_or_options, options = nil, text_or_options if text_or_options.is_a?(Hash)
-      options[:text] ||= text_or_options unless text_or_options.nil?
-      options[:wait] ||= Capybara.default_max_wait_time
-
-      driver.accept_modal(:confirm, options, &blk)
+    def accept_confirm(text = nil, **options, &blk)
+      accept_modal(:confirm, text, options, &blk)
     end
 
     ##
@@ -620,12 +633,8 @@ module Capybara
     #
     # @macro modal_params
     #
-    def dismiss_confirm(text_or_options=nil, options={}, &blk)
-      text_or_options, options = nil, text_or_options if text_or_options.is_a?(Hash)
-      options[:text] ||= text_or_options unless text_or_options.nil?
-      options[:wait] ||= Capybara.default_max_wait_time
-
-      driver.dismiss_modal(:confirm, options, &blk)
+    def dismiss_confirm(text = nil, **options, &blk)
+      dismiss_modal(:confirm, text, options, &blk)
     end
 
     ##
@@ -635,12 +644,8 @@ module Capybara
     # @macro modal_params
     # @option options [String] :with   Response to provide to the prompt
     #
-    def accept_prompt(text_or_options=nil, options={}, &blk)
-      text_or_options, options = nil, text_or_options if text_or_options.is_a?(Hash)
-      options[:text] ||= text_or_options unless text_or_options.nil?
-      options[:wait] ||= Capybara.default_max_wait_time
-
-      driver.accept_modal(:prompt, options, &blk)
+    def accept_prompt(text = nil, **options, &blk)
+      accept_modal(:prompt, text, options, &blk)
     end
 
     ##
@@ -649,12 +654,8 @@ module Capybara
     #
     # @macro modal_params
     #
-    def dismiss_prompt(text_or_options=nil, options={}, &blk)
-      text_or_options, options = nil, text_or_options if text_or_options.is_a?(Hash)
-      options[:text] ||= text_or_options unless text_or_options.nil?
-      options[:wait] ||= Capybara.default_max_wait_time
-
-      driver.dismiss_modal(:prompt, options, &blk)
+    def dismiss_prompt(text = nil, **options, &blk)
+      dismiss_modal(:prompt, text, options, &blk)
     end
 
     ##
@@ -664,17 +665,15 @@ module Capybara
     #
     # If invoked without arguments it will save file to `Capybara.save_path`
     #   and file will be given randomly generated filename. If invoked with a relative path
-    #   the path will be relative to `Capybara.save_path`, which is different from
-    #   the previous behavior with `Capybara.save_and_open_page_path` where the relative path was
-    #   relative to Dir.pwd
+    #   the path will be relative to `Capybara.save_path`
     #
     # @param [String] path  the path to where it should be saved
     # @return [String]      the path to which the file was saved
     #
     def save_page(path = nil)
-      path = prepare_path(path, 'html')
-      File.write(path, Capybara::Helpers.inject_asset_host(body), mode: 'wb')
-      path
+      prepare_path(path, 'html').tap do |p_path|
+        File.write(p_path, Capybara::Helpers.inject_asset_host(body, host: config.asset_host), mode: 'wb')
+      end
     end
 
     ##
@@ -683,15 +682,12 @@ module Capybara
     #
     # If invoked without arguments it will save file to `Capybara.save_path`
     #   and file will be given randomly generated filename. If invoked with a relative path
-    #   the path will be relative to `Capybara.save_path`, which is different from
-    #   the previous behavior with `Capybara.save_and_open_page_path` where the relative path was
-    #   relative to Dir.pwd
+    #   the path will be relative to `Capybara.save_path`
     #
     # @param [String] path  the path to where it should be saved
     #
     def save_and_open_page(path = nil)
-      path = save_page(path)
-      open_file(path)
+      save_page(path).tap { |s_path| open_file(s_path) }
     end
 
     ##
@@ -700,17 +696,13 @@ module Capybara
     #
     # If invoked without arguments it will save file to `Capybara.save_path`
     #   and file will be given randomly generated filename. If invoked with a relative path
-    #   the path will be relative to `Capybara.save_path`, which is different from
-    #   the previous behavior with `Capybara.save_and_open_page_path` where the relative path was
-    #   relative to Dir.pwd
+    #   the path will be relative to `Capybara.save_path`
     #
     # @param [String] path    the path to where it should be saved
     # @param [Hash] options   a customizable set of options
     # @return [String]        the path to which the file was saved
-    def save_screenshot(path = nil, options = {})
-      path = prepare_path(path, 'png')
-      driver.save_screenshot(path, options)
-      path
+    def save_screenshot(path = nil, **options)
+      prepare_path(path, 'png').tap { |p_path| driver.save_screenshot(p_path, options) }
     end
 
     ##
@@ -719,16 +711,15 @@ module Capybara
     #
     # If invoked without arguments it will save file to `Capybara.save_path`
     #   and file will be given randomly generated filename. If invoked with a relative path
-    #   the path will be relative to `Capybara.save_path`, which is different from
-    #   the previous behavior with `Capybara.save_and_open_page_path` where the relative path was
-    #   relative to Dir.pwd
+    #   the path will be relative to `Capybara.save_path`
     #
     # @param [String] path    the path to where it should be saved
     # @param [Hash] options   a customizable set of options
     #
-    def save_and_open_screenshot(path = nil, options = {})
-      path = save_screenshot(path, options)
-      open_file(path)
+    def save_and_open_screenshot(path = nil, **options)
+      # rubocop:disable Lint/Debugger
+      save_screenshot(path, options).tap { |s_path| open_file(s_path) }
+      # rubocop:enable Lint/Debugger
     end
 
     def document
@@ -753,38 +744,165 @@ module Capybara
     end
 
     def current_scope
-      scopes.last || document
+      scope = scopes.last
+      [nil, :frame].include?(scope) ? document : scope
+    end
+
+    ##
+    #
+    # Yield a block using a specific wait time
+    #
+    def using_wait_time(seconds)
+      if Capybara.threadsafe
+        begin
+          previous_wait_time = config.default_max_wait_time
+          config.default_max_wait_time = seconds
+          yield
+        ensure
+          config.default_max_wait_time = previous_wait_time
+        end
+      else
+        Capybara.using_wait_time(seconds) { yield }
+      end
+    end
+
+    ##
+    #
+    #  Accepts a block to set the configuration options if Capybara.threadsafe == true. Note that some options only have an effect
+    #  if set at initialization time, so look at the configuration block that can be passed to the initializer too
+    #
+    def configure
+      raise 'Session configuration is only supported when Capybara.threadsafe == true' unless Capybara.threadsafe
+
+      yield config
+    end
+
+    def self.instance_created?
+      @@instance_created
+    end
+
+    def config
+      @config ||= if Capybara.threadsafe
+        Capybara.session_options.dup
+      else
+        Capybara::ReadOnlySessionConfig.new(Capybara.session_options)
+      end
     end
 
   private
 
+    @@instance_created = false
+
+    def driver_args(args)
+      args.map { |arg| arg.is_a?(Capybara::Node::Element) ? arg.base : arg }
+    end
+
+    def accept_modal(type, text_or_options, options, &blk)
+      driver.accept_modal(type, modal_options(text_or_options, options), &blk)
+    end
+
+    def dismiss_modal(type, text_or_options, options, &blk)
+      driver.dismiss_modal(type, modal_options(text_or_options, options), &blk)
+    end
+
+    def modal_options(text = nil, **options)
+      options[:text] ||= text unless text.nil?
+      options[:wait] ||= config.default_max_wait_time
+      options
+    end
+
     def open_file(path)
-      begin
-        require "launchy"
-        Launchy.open(path)
-      rescue LoadError
-        warn "File saved to #{path}."
-        warn "Please install the launchy gem to open the file automatically."
-      end
+      require 'launchy'
+      Launchy.open(path)
+    rescue LoadError
+      warn "File saved to #{path}.\nPlease install the launchy gem to open the file automatically."
     end
 
     def prepare_path(path, extension)
-      if Capybara.save_path || Capybara.save_and_open_page_path.nil?
-        path = File.expand_path(path || default_fn(extension), Capybara.save_path)
-      else
-        path = File.expand_path(default_fn(extension), Capybara.save_and_open_page_path) if path.nil?
+      File.expand_path(path || default_fn(extension), config.save_path).tap do |p_path|
+        FileUtils.mkdir_p(File.dirname(p_path))
       end
-      FileUtils.mkdir_p(File.dirname(path))
-      path
     end
 
     def default_fn(extension)
-      timestamp = Time.new.strftime("%Y%m%d%H%M%S")
-      path = "capybara-#{timestamp}#{rand(10**10)}.#{extension}"
+      timestamp = Time.new.strftime('%Y%m%d%H%M%S')
+      "capybara-#{timestamp}#{rand(10**10)}.#{extension}"
     end
 
     def scopes
       @scopes ||= [nil]
     end
+
+    def element_script_result(arg)
+      case arg
+      when Array
+        arg.map { |subarg| element_script_result(subarg) }
+      when Hash
+        arg.each { |key, value| arg[key] = element_script_result(value) }
+      when Capybara::Driver::Node
+        Capybara::Node::Element.new(self, arg, nil, nil)
+      else
+        arg
+      end
+    end
+
+    def server_url
+      "http#{'s' if @server.using_ssl?}://#{@server.host}:#{@server.port}" if @server
+    end
+
+    def adjust_server_port(uri)
+      uri.port ||= @server.port if @server && config.always_include_port
+    end
+
+    def _find_frame(*args)
+      return find(:frame) if args.length.zero?
+
+      case args[0]
+      when Capybara::Node::Element
+        args[0]
+      when String, Hash
+        find(:frame, *args)
+      when Symbol
+        find(*args)
+      when Integer
+        idx = args[0]
+        all(:frame, minimum: idx + 1)[idx]
+      else
+        raise TypeError
+      end
+    end
+
+    def _switch_to_window(window = nil, **options, &window_locator)
+      raise Capybara::ScopeError, 'Window cannot be switched inside a `within_frame` block' if scopes.include?(:frame)
+      raise Capybara::ScopeError, 'Window cannot be switched inside a `within` block' unless scopes.last.nil?
+
+      if window
+        driver.switch_to_window(window.handle)
+        window
+      else
+        synchronize_windows(options) do
+          original_window_handle = driver.current_window_handle
+          begin
+            _switch_to_window_by_locator(&window_locator)
+          rescue StandardError
+            driver.switch_to_window(original_window_handle)
+            raise
+          end
+        end
+      end
+    end
+
+    def _switch_to_window_by_locator
+      driver.window_handles.each do |handle|
+        driver.switch_to_window handle
+        return Window.new(self, handle) if yield
+      end
+      raise Capybara::WindowError, 'Could not find a window matching block/lambda'
+    end
+
+    def synchronize_windows(options, &block)
+      wait_time = Capybara::Queries::BaseQuery.wait(options, config.default_max_wait_time)
+      document.synchronize(wait_time, errors: [Capybara::WindowError], &block)
+    end
   end
 end
