@@ -1,21 +1,31 @@
 var activeRequests = 0;
 $(function() {
   $('#change').text('I changed it');
-  $('#drag').draggable();
-  $('#drop').droppable({
+  $('#drag, #drag_scroll, #drag_link').draggable();
+  $('#drop, #drop_scroll').droppable({
     drop: function(event, ui) {
       ui.draggable.remove();
       $(this).html('Dropped!');
     }
   });
-  $('#clickable').click(function() {
+  $('#drag_html5, #drag_html5_scroll').on('dragstart', function(ev){
+    ev.originalEvent.dataTransfer.setData("text", ev.target.id);
+  });
+  $('#drop_html5, #drop_html5_scroll').on('dragover', function(ev){
+    if ($(this).hasClass('drop')) { ev.preventDefault(); }
+  });
+  $('#drop_html5, #drop_html5_scroll').on('drop', function(ev){
+    ev.preventDefault();
+    $(this).html('HTML5 Dropped ' + ev.originalEvent.dataTransfer.getData("text"));
+  });
+  $('#clickable').click(function(e) {
     var link = $(this);
     setTimeout(function() {
       $(link).after('<a id="has-been-clicked" href="#">Has been clicked</a>');
       $(link).after('<input type="submit" value="New Here">');
       $(link).after('<input type="text" id="new_field">');
       $('#change').remove();
-    }, 500);
+    }, 1000);
     return false;
   });
   $('#slow-click').click(function() {
@@ -37,6 +47,9 @@ $(function() {
   $('#with_change_event').change(function() {
     $('body').append($('<p class="change_event_triggered"></p>').text(this.value));
   });
+  $('#with_change_event').on('input', function() {
+    $('body').append($('<p class="input_event_triggered"></p>').text(this.value));
+  });
   $('#checkbox_with_event').click(function() {
     $('body').append('<p id="checkbox_event_triggered">Checkbox event triggered</p>');
   });
@@ -58,15 +71,41 @@ $(function() {
   $('#change-title').click(function() {
     setTimeout(function() {
       $('title').text('changed title')
-    }, 250)
+    }, 400)
+  });
+  $('#change-size').click(function() {
+    setTimeout(function() {
+      document.getElementById('change').style.fontSize = '50px';
+    }, 500)
   });
   $('#click-test').on({
-    dblclick: function() {
-      $(this).after('<a id="has-been-double-clicked" href="#">Has been double clicked</a>');
+    click: function(e) {
+      var desc = "";
+      if (e.altKey) desc += 'alt ';
+      if (e.ctrlKey) desc += 'control ';
+      if (e.metaKey) desc += 'meta ';
+      if (e.shiftKey) desc += 'shift ';
+      var pos = this.getBoundingClientRect();
+      $(this).after('<a id="has-been-clicked" href="#">Has been ' + desc + 'clicked at ' + (e.clientX - pos.left) + ',' + (e.clientY - pos.top) + '</a>');
+    },
+    dblclick: function(e) {
+      var desc = "";
+      if (e.altKey) desc += 'alt ';
+      if (e.ctrlKey) desc += 'control ';
+      if (e.metaKey) desc += 'meta ';
+      if (e.shiftKey) desc += 'shift ';
+      var pos = this.getBoundingClientRect();
+      $(this).after('<a id="has-been-double-clicked" href="#">Has been ' + desc + 'double clicked at ' + (e.clientX - pos.left) + ',' + (e.clientY - pos.top) + '</a>');
     },
     contextmenu: function(e) {
       e.preventDefault();
-      $(this).after('<a id="has-been-right-clicked" href="#">Has been right clicked</a>');
+      var desc = "";
+      if (e.altKey) desc += 'alt ';
+      if (e.ctrlKey) desc += 'control ';
+      if (e.metaKey) desc += 'meta ';
+      if (e.shiftKey) desc += 'shift ';
+      var pos = this.getBoundingClientRect();
+      $(this).after('<a id="has-been-right-clicked" href="#">Has been ' + desc + 'right clicked at ' + (e.clientX - pos.left) + ',' + (e.clientY - pos.top) + '</a>');
     }
   });
   $('#open-alert').click(function() {
@@ -87,6 +126,10 @@ $(function() {
       $(link).attr('opened', 'true');
     }, 3000);
   });
+  $('#alert-page-change').click(function() {
+    alert('Page is changing');
+    return true;
+  });
   $('#open-confirm').click(function() {
     if(confirm('Confirm opened')) {
       $(this).attr('confirmed', 'true');
@@ -102,25 +145,40 @@ $(function() {
       $(this).attr('response', response);
     }
   });
+  $('#open-prompt-with-default').click(function() {
+    var response = prompt('Prompt opened', 'Default value!');
+    if(response === null) {
+      $(this).attr('response', 'dismissed');
+    } else {
+      $(this).attr('response', response);
+    }
+  });
   $('#open-twice').click(function() {
     if (confirm('Are you sure?')) {
       if (!confirm('Are you really sure?')) {
         $(this).attr('confirmed', 'false');
       }
     }
-  })
+  });
   $('#delayed-page-change').click(function() {
     setTimeout(function() {
       window.location.pathname = '/with_html'
     }, 500)
-  })
+  });
   $('#with-key-events').keydown(function(e){
     $('#key-events-output').append('keydown:'+e.which+' ')
   });
   $('#disable-on-click').click(function(e){
-    var input = this
+    var input = this;
     setTimeout(function() {
       input.disabled = true;
     }, 500)
+  });
+  $('#set-storage').click(function(e){
+    sessionStorage.setItem('session', 'session_value');
+    localStorage.setItem('local', 'local value');
+  });
+  $('#multiple-file').change(function(e){
+    $('body').append($('<p class="file_change"input_event_triggered">File input changed</p>'));
   })
 });
