@@ -1,15 +1,17 @@
+# frozen_string_literal: true
+
 Capybara::SpecHelper.spec '#has_table?' do
   before do
     @session.visit('/tables')
   end
 
-  it "should be true if the table is on the page" do
+  it 'should be true if the table is on the page' do
     expect(@session).to have_table('Villain')
     expect(@session).to have_table('villain_table')
-    expect(@session).to have_table(:'villain_table')
+    expect(@session).to have_table(:villain_table)
   end
 
-  it "should be false if the table is not on the page" do
+  it 'should be false if the table is not on the page' do
     expect(@session).not_to have_table('Monkey')
   end
 end
@@ -19,12 +21,12 @@ Capybara::SpecHelper.spec '#has_no_table?' do
     @session.visit('/tables')
   end
 
-  it "should be false if the table is on the page" do
+  it 'should be false if the table is on the page' do
     expect(@session).not_to have_no_table('Villain')
     expect(@session).not_to have_no_table('villain_table')
   end
 
-  it "should be true if the table is not on the page" do
+  it 'should be true if the table is not on the page' do
     expect(@session).to have_no_table('Monkey')
   end
 end
