@@ -1,6 +1,7 @@
 # frozen_string_literal: true
-Capybara::SpecHelper.spec '#become_closed', requires: [:windows, :js] do
-  before(:each) do
+
+Capybara::SpecHelper.spec '#become_closed', requires: %i[windows js] do
+  before do
     @window = @session.current_window
     @session.visit('/with_windows')
     @other_window = @session.window_opened_by do
@@ -8,7 +9,7 @@ Capybara::SpecHelper.spec '#become_closed', requires: [:windows, :js] do
     end
   end
 
-  after(:each) do
+  after do
     @session.document.synchronize(5, errors: [Capybara::CapybaraError]) do
       raise Capybara::CapybaraError if @session.windows.size != 1
     end
@@ -21,18 +22,18 @@ Capybara::SpecHelper.spec '#become_closed', requires: [:windows, :js] do
         @session.execute_script('setTimeout(function(){ window.close(); }, 500);')
       end
       Capybara.using_wait_time 0.1 do
-        expect(@other_window).to become_closed(wait: 2)
+        expect(@other_window).to become_closed(wait: 5)
       end
     end
 
     it 'should raise error if value of :wait is less than timeout' do
       @session.within_window @other_window do
-        @session.execute_script('setTimeout(function(){ window.close(); }, 700);')
+        @session.execute_script('setTimeout(function(){ window.close(); }, 1000);')
       end
       Capybara.using_wait_time 2 do
         expect do
-          expect(@other_window).to become_closed(wait: 0.4)
-        end.to raise_error(RSpec::Expectations::ExpectationNotMetError, /\Aexpected #<Window @handle=".+"> to become closed after 0.4 seconds\Z/)
+          expect(@other_window).to become_closed(wait: 0.2)
+        end.to raise_error(RSpec::Expectations::ExpectationNotMetError, /\Aexpected #<Window @handle=".+"> to become closed after 0.2 seconds\Z/)
       end
     end
   end
@@ -42,7 +43,7 @@ Capybara::SpecHelper.spec '#become_closed', requires: [:windows, :js] do
       @session.within_window @other_window do
         @session.execute_script('setTimeout(function(){ window.close(); }, 500);')
       end
-      Capybara.using_wait_time 1.5 do
+      Capybara.using_wait_time 5 do
         expect(@other_window).to become_closed
       end
     end
@@ -60,25 +61,25 @@ Capybara::SpecHelper.spec '#become_closed', requires: [:windows, :js] do
   end
 
   context 'with not_to' do
-    it 'should raise error if default_max_wait_time is more than timeout' do
+    it "should not raise error if window doesn't close before default_max_wait_time" do
       @session.within_window @other_window do
-        @session.execute_script('setTimeout(function(){ window.close(); }, 700);')
+        @session.execute_script('setTimeout(function(){ window.close(); }, 1000);')
       end
-      Capybara.using_wait_time 0.4 do
+      Capybara.using_wait_time 0.3 do
         expect do
           expect(@other_window).not_to become_closed
-        end
+        end.not_to raise_error
       end
     end
 
-    it 'should raise error if default_max_wait_time is more than timeout' do
+    it 'should raise error if window closes before default_max_wait_time' do
       @session.within_window @other_window do
         @session.execute_script('setTimeout(function(){ window.close(); }, 700);')
       end
-      Capybara.using_wait_time 1.1 do
+      Capybara.using_wait_time 3.1 do
         expect do
           expect(@other_window).not_to become_closed
-        end.to raise_error(RSpec::Expectations::ExpectationNotMetError, /\Aexpected #<Window @handle=".+"> not to become closed after 1.1 seconds\Z/)
+        end.to raise_error(RSpec::Expectations::ExpectationNotMetError, /\Aexpected #<Window @handle=".+"> not to become closed after 3.1 seconds\Z/)
       end
     end
   end
