@@ -1,34 +1,35 @@
 # frozen_string_literal: true
+
 Capybara::SpecHelper.spec '#within' do
   before do
     @session.visit('/with_scope')
   end
 
-  context "with CSS selector" do
-    it "should click links in the given scope" do
-      @session.within(:css, "#for_bar li", text: 'With Simple HTML') do
+  context 'with CSS selector' do
+    it 'should click links in the given scope' do
+      @session.within(:css, '#for_bar li:first-child') do
         @session.click_link('Go')
       end
       expect(@session).to have_content('Bar')
     end
 
-    it "should assert content in the given scope" do
-      @session.within(:css, "#for_foo") do
+    it 'should assert content in the given scope' do
+      @session.within(:css, '#for_foo') do
         expect(@session).not_to have_content('First Name')
       end
       expect(@session).to have_content('First Name')
     end
 
-    it "should accept additional options" do
-      @session.within(:css, "#for_bar li", text: 'With Simple HTML') do
+    it 'should accept additional options' do
+      @session.within(:css, '#for_bar li', text: 'With Simple HTML') do
         @session.click_link('Go')
       end
       expect(@session).to have_content('Bar')
     end
   end
 
-  context "with XPath selector" do
-    it "should click links in the given scope" do
+  context 'with XPath selector' do
+    it 'should click links in the given scope' do
       @session.within(:xpath, "//div[@id='for_bar']//li[contains(.,'With Simple HTML')]") do
         @session.click_link('Go')
       end
@@ -36,8 +37,8 @@ Capybara::SpecHelper.spec '#within' do
     end
   end
 
-  context "with the default selector" do
-    it "should use XPath" do
+  context 'with the default selector' do
+    it 'should use XPath' do
       @session.within("//div[@id='for_bar']//li[contains(.,'With Simple HTML')]") do
         @session.click_link('Go')
       end
@@ -45,9 +46,9 @@ Capybara::SpecHelper.spec '#within' do
     end
   end
 
-  context "with Node rather than selector" do
-    it "should click links in the given scope" do
-      node_of_interest = @session.find(:css, "#for_bar li", text: 'With Simple HTML')
+  context 'with Node rather than selector' do
+    it 'should click links in the given scope' do
+      node_of_interest = @session.find(:css, '#for_bar li', text: 'With Simple HTML')
 
       @session.within(node_of_interest) do
         @session.click_link('Go')
@@ -56,19 +57,21 @@ Capybara::SpecHelper.spec '#within' do
     end
   end
 
-  context "with the default selector set to CSS" do
+  context 'with the default selector set to CSS' do
     before { Capybara.default_selector = :css }
-    it "should use CSS" do
-      @session.within("#for_bar li", text: 'With Simple HTML') do
+
+    after { Capybara.default_selector = :xpath }
+
+    it 'should use CSS' do
+      @session.within('#for_bar li', text: 'With Simple HTML') do
         @session.click_link('Go')
       end
       expect(@session).to have_content('Bar')
     end
-    after { Capybara.default_selector = :xpath }
   end
 
-  context "with nested scopes" do
-    it "should respect the inner scope" do
+  context 'with nested scopes' do
+    it 'should respect the inner scope' do
       @session.within("//div[@id='for_bar']") do
         @session.within(".//li[contains(.,'Bar')]") do
           @session.click_link('Go')
@@ -77,7 +80,7 @@ Capybara::SpecHelper.spec '#within' do
       expect(@session).to have_content('Another World')
     end
 
-    it "should respect the outer scope" do
+    it 'should respect the outer scope' do
       @session.within("//div[@id='another_foo']") do
         @session.within(".//li[contains(.,'With Simple HTML')]") do
           @session.click_link('Go')
@@ -87,14 +90,14 @@ Capybara::SpecHelper.spec '#within' do
     end
   end
 
-  it "should raise an error if the scope is not found on the page" do
+  it 'should raise an error if the scope is not found on the page' do
     expect do
       @session.within("//div[@id='doesnotexist']") do
       end
     end.to raise_error(Capybara::ElementNotFound)
   end
 
-  it "should restore the scope when an error is raised" do
+  it 'should restore the scope when an error is raised' do
     expect do
       @session.within("//div[@id='for_bar']") do
         expect do
@@ -102,12 +105,12 @@ Capybara::SpecHelper.spec '#within' do
             @session.within(".//div[@id='doesnotexist']") do
             end
           end.to raise_error(Capybara::ElementNotFound)
-        end.to_not change { @session.has_xpath?(".//div[@id='another_foo']") }.from(false)
+        end.not_to change { @session.has_xpath?(".//div[@id='another_foo']") }.from(false)
       end
-    end.to_not change { @session.has_xpath?(".//div[@id='another_foo']") }.from(true)
+    end.not_to change { @session.has_xpath?(".//div[@id='another_foo']") }.from(true)
   end
 
-  it "should fill in a field and click a button" do
+  it 'should fill in a field and click a button' do
     @session.within("//li[contains(.,'Bar')]") do
       @session.click_button('Go')
     end
@@ -119,6 +122,13 @@ Capybara::SpecHelper.spec '#within' do
     end
     expect(extract_results(@session)['first_name']).to eq('Dagobert')
   end
+
+  it 'should have #within_element as an alias' do
+    expect(Capybara::Session.instance_method(:within)).to eq Capybara::Session.instance_method(:within_element)
+    @session.within_element(:css, '#for_foo') do
+      expect(@session).not_to have_content('First Name')
+    end
+  end
 end
 
 Capybara::SpecHelper.spec '#within_fieldset' do
@@ -126,18 +136,18 @@ Capybara::SpecHelper.spec '#within_fieldset' do
     @session.visit('/fieldsets')
   end
 
-  it "should restrict scope to a fieldset given by id" do
-    @session.within_fieldset("villain_fieldset") do
-      @session.fill_in("Name", with: 'Goldfinger')
-      @session.click_button("Create")
+  it 'should restrict scope to a fieldset given by id' do
+    @session.within_fieldset('villain_fieldset') do
+      @session.fill_in('Name', with: 'Goldfinger')
+      @session.click_button('Create')
     end
     expect(extract_results(@session)['villain_name']).to eq('Goldfinger')
   end
 
-  it "should restrict scope to a fieldset given by legend" do
-    @session.within_fieldset("Villain") do
-      @session.fill_in("Name", with: 'Goldfinger')
-      @session.click_button("Create")
+  it 'should restrict scope to a fieldset given by legend' do
+    @session.within_fieldset('Villain') do
+      @session.fill_in('Name', with: 'Goldfinger')
+      @session.click_button('Create')
     end
     expect(extract_results(@session)['villain_name']).to eq('Goldfinger')
   end
@@ -148,18 +158,18 @@ Capybara::SpecHelper.spec '#within_table' do
     @session.visit('/tables')
   end
 
-  it "should restrict scope to a fieldset given by id" do
-    @session.within_table("girl_table") do
-      @session.fill_in("Name", with: 'Christmas')
-      @session.click_button("Create")
+  it 'should restrict scope to a fieldset given by id' do
+    @session.within_table('girl_table') do
+      @session.fill_in('Name', with: 'Christmas')
+      @session.click_button('Create')
     end
     expect(extract_results(@session)['girl_name']).to eq('Christmas')
   end
 
-  it "should restrict scope to a fieldset given by legend" do
-    @session.within_table("Villain") do
-      @session.fill_in("Name", with: 'Quantum')
-      @session.click_button("Create")
+  it 'should restrict scope to a fieldset given by legend' do
+    @session.within_table('Villain') do
+      @session.fill_in('Name', with: 'Quantum')
+      @session.click_button('Create')
     end
     expect(extract_results(@session)['villain_name']).to eq('Quantum')
   end
